/**
 * @file debugging.cs 
 * Classes to aid debugging (logger for now).
 *
 * (C) 2004 Martin Pitt <martin@piware.de>
 *
 * This software is distributed under the terms and conditions of the GNU
 * General Public License (see file COPYING)
 */

using System;
using System.IO;

namespace Debugging {

public class Logger {
    /// Initialize logger that writes to the given TextWriter. Sets initial
    /// indentation step to 4.
    public Logger( TextWriter writer ) {
	this.writer = writer;
	_level = 0;
	_indent = 4;
    }

    /// Initialize logger to standard output. Sets initial indentation step to
    /// 4.
    public Logger() 
	: this( Console.Out )
    {}

    /// Indentation step property (4 by default).
    public int indent {
	get { return _indent; }
	set { _indent = value; }
    }

    /// Return current indentation level.
    public int level {
	get { return _level; }
    }

    /// Increment indentation level.
    public static Logger operator ++( Logger l ) {
	++l._level;
	return l;
    }

    /// Decrement indentation level.
    public static Logger operator --( Logger l ) {
	if( l._level == 0 )
	    throw new Exception( "Cannot decrease indentation level, it is already 0" );
	--l._level;
	return l;
    }

    /// Log the given message to the output writer and append a newline.
    public void log( string msg ) {
	doIndent();
	writer.WriteLine( msg );
    }

    /// Log the given message to the output writer and append a newline.
    public void log( string msg, Object arg1 ) {
	doIndent();
	writer.WriteLine( msg, arg1 );
    }

    /// Log the given message to the output writer and append a newline.
    public void log( string msg, Object arg1, Object arg2 ) {
	doIndent();
	writer.WriteLine( msg, arg1, arg2 );
    }

    /// Log the given message to the output writer and append a newline.
    public void log( string msg, Object arg1, Object arg2, Object arg3 ) {
	doIndent();
	writer.WriteLine( msg, arg1, arg2, arg3 );
    }

    /// Log the given message to the output writer and append a newline.
    public void log( string msg, Object[] args ) {
	doIndent();
	writer.WriteLine( msg, args );
    }

    /// Write indentation to writer.
    private void doIndent() {
	for( int i = _indent * _level; i > 0; --i )
	    writer.Write( " " );
    }

    private TextWriter writer;
    private int _indent, _level;
}

}
